{% load i18n %}
{% spaceless %}
<div class="cascade-colorpicker cascade-color-picker">
	{% with widget=widget.subwidgets.0 %}{% include widget.template_name %}{% endwith %}
	{% with widget=widget.subwidgets.1 %}{% include widget.template_name %}{% endwith %}
	<em>{% trans "Inherit" %}</em>
	{% with widget=widget.subwidgets.2 %}{% include widget.template_name %}{% endwith %}
</div>
{% endspaceless %}
