{% with css_classes=instance.css_classes inline_styles=instance.inline_styles %}
<{{ instance.tag_type }}{% if css_classes %} class="{{ css_classes }}"{% endif %}{% if inline_styles %} style="{{ inline_styles }}"{% endif %} />
{% endwith %}