{% spaceless %}{% with instance_link=instance.link instance_css_classes=instance.css_classes download=instance.download_name instance_inline_styles=instance.inline_styles %}
{% block link_link %}
	{% if instance_link %}<a href="{{ instance_link }}"{{ instance.html_tag_attributes }}{% if instance_css_classes %} class="{{ instance_css_classes }}"{% endif %}{% if download %} download="{{ download }}" target="_blank"{% endif %}{% if instance_inline_styles %} style="{{ instance_inline_styles }}"{% endif %}>{% endif %}{% block link_content %}{{ instance.content }}{% endblock %}{% if instance_link %}</a>{% endif %}
{% endblock %}
{% endwith %}{% endspaceless %}{# NEVER ADD A FINAL NEWLINE #}