django.jQuery(function($) {
	// create class handling the client-side part of a TextLinkPlugin
	django.cascade.TextLinkPlugin = ring.create(eval(django.cascade.ring_plugin_bases.TextLinkPlugin), {});
});
