# Generated by Django 4.0.6.dev20220601124058 on 2022-08-02 08:54

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cmsplugin_cascade', '0030_separate_ids_per_language'),
    ]

    operations = [
        migrations.AlterField(
            model_name='texteditorconfigfields',
            name='element_type',
            field=models.CharField(choices=[('p', 'p'), ('h1', 'h1'), ('h2', 'h2'), ('h3', 'h3'), ('h4', 'h4'), ('h5', 'h5'), ('h6', 'h6'), ('pre', 'pre'), ('address', 'address'), ('div', 'div'), ('span', 'span'), ('ol', 'ol'), ('ul', 'ul')], max_length=12, verbose_name='Element Type'),
        ),
    ]
