{# instead of throwing an exception, just render a warning message #}
<div class="alert alert-danger" role="alert">The chosen template '{{ instance.glossary.render_template }}' could not be found.</div>