{% spaceless %}
<div class="cascade-colorpicker cascade-border-choice">
	{% with widget=widget.subwidgets.0 %}{% include widget.template_name %}{% endwith %}<em></em>
	{% with widget=widget.subwidgets.1 %}{% include widget.template_name %}{% endwith %}<em></em>
	{% with widget=widget.subwidgets.2 %}{% include widget.template_name %}{% endwith %}
	{% with widget=widget.subwidgets.3 %}{% include widget.template_name %}{% endwith %}
</div>
{% endspaceless %}
