{% load static sekizai_tags %}{% spaceless %}
{% if stylesheet_url %}{% addtoblock "css" %}<link href="{{ stylesheet_url }}" rel="stylesheet" type="text/css" />{% endaddtoblock %}{% endif %}
{% with instance_link=instance.link %}
{% if instance_link %}<a href="{{ instance_link }}"{{ instance.html_tag_attributes }}>{% endif %}<i class="{{ icon_font_class }}">{% if not icon_font_class %}{# render a placeholder for icon selection #} &square; {% endif %}</i>{% if instance_link %}</a>{% endif %}
{% endwith %}
{% endspaceless %}
